page	75, 110

;--------------------------------------------------------------------
;
;	SYSID.ASM
;
;	Version 4.3
;
;	Two subprograms used by SYSID.PAS:
;
;		DISKREAD	- reads absolute sectors from disk
;					(function)
;		CPUID		- identifies host CPU and coprocessor
;					(function)
;
;	Steve Grant
;	Long Beach, CA
;	December 9, 1988
;
;--------------------------------------------------------------------

.8086
.8087

	public	DISKREAD, CPUID, NDPID

CODE	segment	byte

;	Conditional jumps are all coded with the SHORT qualifier in
;	order to minimize the size of the .OBJ file output of Turbo
;	Assembler.

;--------------------------------------------------------------------

assume cs:CODE, ds:nothing, es:nothing

DISKREAD	proc	near

;	On entry:
;
;		[SP+10]		= drive number (0=A, 1=B, etc.)
;		[SP+8]		= starting logical sector number
;		[SP+6]		= number of sectors to read
;		[SP+4]:[SP+2]	= far address of disk transfer area
;		[SP]		= near return address
;
;	On exit:
;
;		AX	= function result
;			00	- function successful
;			01..FF	- DOS INT 25H error result

	drive			equ	[bp+10]
	starting_sector		equ	[bp+8]
	number_of_sectors	equ	[bp+6]
	buffer			equ	[bp+2]

	push	bp
	mov	bp,sp
	add	bp,2			; PUSH BP
	push	ds
	mov	al,drive
	mov	dx,starting_sector
	mov	cx,number_of_sectors
	lds	bx,buffer
	int	25H
	jc	short diskread_01
	xor	ax,ax
diskread_01:

; fix broken stack

	pop	bp

; clean up and go home

	pop	ds
	pop	bp
	ret	10

DISKREAD	endp

;--------------------------------------------------------------------

assume	cs:CODE, ds:DATA, es:nothing

CPUID	proc	near

;	On exit:
;	
;		AH	= CPU type
;		AL	= segment register change/interrupt flag

f8088	equ	0
f8086	equ	1
fV20	equ	2
fV30	equ	3
f80188	equ	4
f80186	equ	5
f80286	equ	6
f80386	equ	7
funk	=	0FFH

false	equ	0
true	equ	1

	call	cpu
	push	ax
	call	chkint
	pop	ax
	mov	al,bh
	ret

;--------------------------------------------------------------------

cpu:

;	On exit:
;
;		AH	= CPU type (f8088..f80386, funk)

; interrupt of multi-prefix string instruction

	sti
	mov	cx,0FFFFH
	rep lods	byte ptr es:[si]
	jcxz	short cpu_02
	call	piq
	cmp	bx,4
	jne	short cpu_01
	mov	ah,f8088
	ret
cpu_01:
	cmp	bx,6
	jne	short cpu_08
	mov	ah,f8086
	ret
cpu_02:

; number of bits in displacement register used by shift

	mov	al,0FFH
	mov	cl,20H
	shl	al,cl
	or	al,al
	jnz	short cpu_04
	call	piq
	cmp	bx,4
	jne	short cpu_03
	mov	ah,fV20
	ret
cpu_03:
	cmp	bx,6
	jne	short cpu_08
	mov	ah,fV30
	ret
cpu_04:

; order of write/decrement by PUSH SP

	push	sp
	pop	ax
	cmp	ax,sp
	je	short cpu_06
	call	piq
	cmp	bx,4
	jne	short cpu_05
	mov	ah,f80188
	ret
cpu_05:
	cmp	bx,6
	jne	short cpu_08
	mov	ah,f80186
	ret
cpu_06:

; try to alter flag register bits 15-12

	pushf
	pop	ax
	mov	bx,ax
	xor	bx,0F000H
	push	bx
	popf
	pushf
	pop	bx
	cmp	ax,bx
	jne	short cpu_07
	mov	ah,f80286
	ret
cpu_07:
	mov	ah,f80386
	ret
cpu_08:
	mov	ah,funk
	ret

CPUID	endp

;--------------------------------------------------------------------

piq:

; 	On exit:
;
;		BX	= length of prefetch instruction queue
;
;	This subroutine uses self-modifying code, but can
;	nevertheless be run repeatedly in the course of the calling
;	program.

count	equ	7
opincbx	equ	43H			; inc bx opcode
opnop	equ	90H			; nop opcode

	mov	al,opincbx
	mov	cx,count
	push	cx
	push	cs
	pop	es
	mov	di,offset piq_01 - 1
	push	di
	std
	rep stosb
	mov	al,opnop
	pop	di
	pop	cx
	xor	bx,bx
	cli
	rep stosb
	rept	count
	inc	bx
	endm
piq_01:
	sti
	ret

;--------------------------------------------------------------------

chkint:

;	On entry:
;
;		AH	= CPU type
;
;	On exit:
;
;		BH	= 0 if CPU does not acknowledge interrupts
;				immediately after segment register
;				change
;			= 1 if CPU acknowledges interrupts
;				immediately after segment register
;				change

; does this CPU need checking?

	cmp	ah,f80386
	ja	short chkint_ret

; save old INT 01H vector

	mov	ax,3501H
	int	21H
	mov	old_int01_ofs,bx
	mov	old_int01_seg,es

; redirect INT 01H vector

	push	ds
	mov	ax,2501H
	mov	dx,seg new_int01
	mov	ds,dx
	mov	dx,offset new_int01
	int	21H
	pop	ds

; set TF and change SS -- did we trap on following instruction?

	pushf
	pop	ax
	or	ah,01H			; set TF
	push	ax
	popf
	push	ss			; CPU may wait one
					; instruction before
					; recognizing single step
					; interrupt
	pop	ss
chkint_01:				; shouldn't ever trap here

; restore old INT 01H vector

	push	ds
	mov	ax,2501H
	lds	dx,old_int01
	int	21H
	pop	ds
chkint_ret:
	ret

;--------------------------------------------------------------------

new_int01:

;	INT 01H handler (single step)
;
;	On entry:
;
;		[SP]	= IP
;		[SP+2]	= CS
;		[SP+4]	= flags

	pop	ax			; IP
	cmp	ax,offset chkint_01
	jb	short new_int01_03
	je	short new_int01_01
	mov	bh,false
	jmp	short new_int01_02
new_int01_01:
	mov	bh,true
new_int01_02:
	pop	cx			; CS
	pop	dx			; flags
	and	dh,0FEH			; turn off TF
	push	dx			; flags
	push	cx			; CS
new_int01_03:
	push	ax			; IP
	iret

;--------------------------------------------------------------------

assume	cs:CODE, ds:DATA, es:nothing

NDPID	proc	near

;	On exit:
;
;		DX	= NDP type (fnone..f80387, funk)
;		AX	= NDP control word

fnone	equ	0
f8087	equ	1
f80287	equ	2
f80387	equ	3
funk	=	0FFH

	fsave	ndp_save
	finit
	fstcw	ndp_cw
	frstor	ndp_save
	fwait
	cmp	ndp_cw,0000H
	jne	short ndp_01
	mov	dx,fnone
	jmp	short ndp_04
ndp_01:
	cmp	ndp_cw,03FFH
	jne	short ndp_02
	mov	dx,f8087
	jmp	short ndp_04
ndp_02:
	cmp	ndp_cw,037FH
	jne	short ndp_03
	mov	dx,f80287
	jmp	short ndp_04
ndp_03:
	mov	dx,funk
	ret
ndp_04:
	mov	ax,ndp_save		; CW is first word of 80x87
					; save area
	ret

NDPID	endp

CODE	ends

;--------------------------------------------------------------------

DATA	segment	byte

; storage for CPUID

; redirected INT 01H vector

old_int01	label	dword
old_int01_ofs	dw	?
old_int01_seg	dw	?

; storage for NDPID

; 80x87 environment array, control word

ndp_save	dw	94 dup (?)
ndp_cw		dw	?

DATA	ends

	end
