(*
**	SYSID.INC
**
**	Version 4.3
**
**	The user functions and procedures for SYSID.PAS.
**
**	Steve Grant
**	Long Beach, CA
**	December 9, 1988
*)

(*$L SYSID*)

function diskread(drive : byte; starting_sector, number_of_sectors : word
	; var buffer) : word;

(*
**	Returns 0 if no error, else the error value from DOS.
*)

external;

procedure rjustify(a : string);

var
	i : byte;

begin
	for i := wherex to twidth - 1 - length(a) do
		write(' ');
	write(a)
end;

procedure border;

const
	ch = '';

var
	i : byte;

begin
	for i := 1 to twidth - 1 do
		write(ch)
end;

procedure caption1(a : string);

begin
	textcolor(lightgray);
	write(a);
	textcolor(lightgreen)
end;

procedure caption2(a : string);

const
	capterm = ': ';

var
	i : byte;
	xbool : boolean;

begin
	i := length(a);
	while (i > 0) and (a[i] = ' ') do
		dec(i);
	insert(capterm, a, i + 1);
	caption1(a)
end;

function nocarry : boolean;

begin
	nocarry := regs.flags and fcarry = $0000
end;

function hex(a : word; b : byte) : string;

const
	digit : array[$0..$F] of char = '0123456789ABCDEF';

var
	i : byte;
	xstring : string;

begin
	xstring := '';
	for i := 1 to b do begin
		insert(digit[a and $000F], xstring, 1);
		a := a shr 4
	end;
	hex := xstring
end;

procedure unknown(a : string; b : word; c : byte);

begin
	writeln('(unknown', ' ', a, ' ', hex(b, c), ')')
end;

function bin4(a : byte) : string;

const
	digit : array[0..1] of char = '01';

var
	xstring : string;
	i : byte;

begin
	xstring := '';
	for i := 3 downto 0 do begin
		insert(digit[a mod 2], xstring, 1);
		a := a shr 1
	end;
	bin4 := xstring
end;

function bin8(a : byte) : string;

begin
	bin8 := bin4(a shr 4) + '_' + bin4(a and $0F)
end;

function scan(a : string; b, c, d : word; var e : word) : boolean;

var
	i : longint;
	j : byte;
	len : byte;
	xbool1 : boolean;
	xbool2 : boolean;

begin
	i := c;
	len := length(a);
	xbool1 := false;
	repeat
		if i <= longint(d) - len + 1 then begin
			j := 0;
			xbool2 := false;
			repeat
				if j < len then
					if upcase(chr(mem[b : i + j])) = a[j + 1] then
						inc(j)
					else begin
						xbool2 := true;
						inc(i)
					end
				else begin
					xbool2 := true;
					xbool1 := true;
					e := i;
					scan := true
				end
			until xbool2
		end else begin
			xbool1 := true;
			scan := false
		end
	until xbool1
end;

function BIOSscan(a, b, c : word; var d : word) : boolean;

const
	max = 3;
	notice : array[1..max] of string = ('(C)', 'COPR.', 'COPYRIGHT');

var
	i : 1..max;
	len : byte;
	target : string;
	xbool : boolean;
	xlong : longint;
	xword : word;

begin
	xlong := c;
	xbool := false;
	for i := 1 to max do begin
		target := notice[i];
		len := length(target);
		if xbool then
			xlong := longint(xword) - 2 + len;
		if (xlong >= b) and (xlong <= c) and (scan(target, a, b, xlong, xword))
			then
			xbool := true
	end;
	if xbool then begin
		while (xword > b) and (chr(mem[a : xword - 1]) in pchar) do
			dec(xword);
		d := xword
	end;
	BIOSscan := xbool
end;

procedure showBIOS(a, b : word);

var
	xbool : boolean;
	xchar : char;

begin
	xbool := false;
	repeat
		xchar := chr(mem[a : b]);
		if xchar in pchar then begin
			write(xchar);
			if b < $FFFF then
				inc(b)
			else
				xbool := true
		end else
			xbool := true
	until xbool;
	writeln
end;

procedure dontknow;

begin
	writeln('(unknown)')
end;

function CPUID : word;

(*
**	Describes the host CPU.
**
**		High byte:
**
**			00	= 8088
**			01	= 8086
**			02	= V20
**			03	= V30
**			04	= 80188
**			05	= 80186
**			06	= 80286
**			07	= 80386
**
**		Low byte:
**
**			00	= CPU does not acknowledge interrupts immediately after
**					segment register change
**			01	= CPU acknowledges interrupts immediately after segment
**					register change
*)

external;

function NDPID : longint;

(*
**	Describes the host coprocessor.
**
**		DX	= coprocessor type
**
**				00	- none
**				01	- 8087
**				02	- 80287
**				03	- 80387
**
**		AX	= NDP control word
*)

external;

procedure segofs(a, b : word);

begin
	write(hex(a, 4), ':', hex(b, 4))
end;

procedure yesorno(a : boolean);

begin
	if a then
		write('yes')
	else
		write('no ')
end;

function cbw(a, b : byte) : word;

begin
	cbw := a shl 8 + b
end;

procedure showNDP(a : string; b : word);

begin
	writeln(a);
	caption2(qindent + 'Infinity');
	case b and $1000 of
		$0000 : writeln('projective');
		$1000 : writeln('affine')
	end;
	caption2(qindent + 'Rounding');
	case b and $0C00 of
		$0000 : writeln('to nearest or even');
		$0400 : writeln('down');
		$0800 : writeln('up');
		$0C00 : writeln('chop')
	end;
	caption2(qindent + 'Precision');
	case b and $0300 of
		$0000 : writeln('24 bits');
		$0100 : writeln('(reserved)');
		$0200 : writeln('53 bits');
		$0300 : writeln('64 bits')
	end
end;

function intinit(a : byte) : boolean;

begin
	intinit := longint(intvec[a]) <> $00000000
end;

function showchar(a : char) : char;

begin
	if a in pchar then
		showchar := a
	else
		showchar := '.'
end;

procedure EMMerr(a : byte);

begin
	case a of
		$80 : writeln('internal error in EMM software');
		$81 : writeln('malfunction in expanded memory hardware');
		$82 : writeln('memory manager busy');
		$83 : writeln('invalid handle');
		$84 : writeln('undefined function');
		$85 : writeln('no more handles available');
		$86 : writeln('error in save or restore of mapping context');
		$87 : writeln('not enough physical pages available');
		$88 : writeln('not enough free pages available');
		$89 : writeln('no pages requested');
		$8A : writeln('logical page outside range assigned to handle');
		$8B : writeln('invalid physical page number');
		$8C : writeln('page map hardware state save area full');
		$8D : writeln('mapping context already in save area');
		$8E : writeln('mapping context not in save area');
		$8F : writeln('undefined subfunction parameter')
		else
			unknown('expanded memory error', a, 2)
	end
end;

procedure pause;

var
	xbyte : byte;
	xchar : char;

begin
	if wherey + topline = tlength - 1 then begin
		xbyte := textattr;
		textcolor(green);
		write('(Press any key to continue)');
		repeat
		until keypressed;
		while keypressed do
			xchar := readkey;
		clrscr;
		writeln('(continued)');
		textattr := xbyte
	end
end;

procedure showMCB(MCB, ownerPID, parent, size : word);

var
	i : word;
	xbool : boolean;
	xchar : char;
	xlong1 : longint;
	xlong2 : longint;
	xlong3 : longint;
	xstring : string;
	xword : word;

begin
	xlong1 := longint(size) shl 4;
	xword := memw[ownerPID : $002C];
	if ownerPID = $0008 then
		xstring := 'IBMDOS.COM'
	else if ownerPID = parent then
		xstring := 'COMMAND.COM'
(*	BIX ms.dos/secrets #1496  *)
(*	Software Tools #145, p. 56  *)
	else if (ownerPID = $0000) or (ownerPID = prefixseg) then
		xstring := '(free)'
	else begin
		i := 0;
		while memw[xword : i] > $0000 do
			inc(i);
		inc(i, 4);
		xstring := '';
		xbool := false;
		repeat
			xchar := chr(mem[xword : i]);
			if xchar in pchar then begin
				if xchar in dirsep then
					xstring := ''
				else
					xstring := xstring + xchar;
				inc(i)
			end else begin
				xbool := true;
				if xchar > #0 then
					xstring := ''
			end
		until xbool;
	end;
	write(hex(MCB, 4), qspace3, hex(ownerPID, 4), qspace3, hex(parent, 4)
		, '  ', qspace3, xlong1 : 6, qspace3);
	if xword = MCB + 1 then
		write('  ')
	else
		write('   ');
	write(qspace3, xstring);
	if MCB + 1 = ownerPID then begin
		for i := length(xstring) + 1 to 12 do
			write(' ');
		write(qspace3);
		xlong2 := longint(ownerPID) shl 4;
		for i := $00 to $FF do begin
			xlong3 := longint(intvec[i]) and $FFFF0000 shr 12
				+ longint(intvec[i]) and $0000FFFF;
			if (xlong2 <= xlong3) and (xlong3 <= xlong2 + xlong1) then begin
				if wherex > twidth - 3 then begin
					writeln;
					pause;
					write('    ', qspace3, '    ', qspace3, '      ', qspace3
						, '      ',qspace3, '   ', qspace3, '            '
						, qspace3)
				end;
				write(hex(i, 2), ' ');
			end
		end
	end;
	writeln
end;

procedure showdisp(a : string; b : byte);

begin
	caption2(a);
	case b of
		$00 : writeln('(none)');
		$01 : writeln('MDA + 5151');
		$02 : writeln('CGA + 5153/5154');
		$03 : writeln('(reserved)');
		$04 : writeln('EGA + 5153/5154');
		$05 : writeln('EGA 5151');
		$06 : writeln('PGA + 5175');
		$07 : writeln('VGA + analog monochrome');
		$08 : writeln('VGA + analog color');
		$09 : writeln('(reserved)');
		$0A : writeln('MCGA + digital color');
		$0B : writeln('MCGA + digital monochrome');
		$0C : writeln('MCGA + analog color');
		$0D..$FE : writeln('(reserved)');
		$FF : dontknow
	end
end;

procedure showcolor(a : byte);

begin
	case(a) of
		black			: write('black');
		blue			: write('blue');
		green			: write('green');
		cyan			: write('cyan');
		red				: write('red');
		magenta			: write('magenta');
		brown			: write('brown');
		lightgray		: write('light gray');
		darkgray		: write('dark gray');
		lightblue		: write('light blue');
		lightgreen		: write('light green');
		lightcyan		: write('light cyan');
		lightred		: write('light red');
		lightmagenta	: write('light magenta');
		yellow			: write('yellow');
		white			: write('white')
		else
			unknown('color', a, 2)
	end
end;

procedure captfont;

begin
	caption1('Font        ' + qspace3 + 'Address');
	writeln;
	write('INT 1FH     ', qspace3);
	segofs(longint(intvec[$1F]) shr 16, longint(intvec[$1F]) and $0000FFFF);
	writeln
end;

procedure showfont(a : byte);

begin
	with regs do begin
		case a of
			$00 : write('INT 1FH     ');
			$01 : write('INT 43H     ');
			$02 : write('ROM 8x14    ');
			$03 : write('ROM 8x8 (lo)');
			$04 : write('ROM 8x8 (hi)');
			$05 : write('ROM 9x14    ');
			$06 : write('ROM 8x16    ');
			$07 : write('ROM 9x16    ')
		end;
		write(qspace3);
		AX := $1130;
		BH := a;
		intr($10, regs);
		segofs(ES, BP);
		writeln
	end
end;

procedure int101210;

begin
	with regs do begin
		AH := $12;
		BL := $10;
		intr($10, regs);
		caption2('Display type');
		case BH of
			$00 : writeln('color');
			$01 : writeln('monochrome')
			else
				dontknow
		end;
		caption2('Memory');
		case BL of
			$00 : writeln('64K');
			$01 : writeln('128K');
			$02 : writeln('192K');
			$03 : writeln('256K')
			else
				dontknow
		end;
		caption2('Feature bits');
		writeln(bin4(CH and $0F));
		caption2('DIP switches');
		writeln(bin4(CL and $0F))
	end
end;

procedure offoron(a : boolean);

begin
	if a then
		write('on ')
	else
		write('off')
end;

procedure zeropad(a : byte);

begin
	if a < 10 then
		write('0');
	write(a)
end;

procedure showvers;

var
	xchar : char;

begin
	xchar := chr(country[9]);
	if osmajor > 0 then begin
		write(osmajor, xchar);
		zeropad(osminor);
		writeln
	end else
		writeln('1', xchar, 'x')
end;

procedure showecho(a : word);

var
	xbyte : byte;

begin
	xbyte := mem[DOScseg : a];
	case xbyte of
		$00 : writeln('off');
		$FF : writeln('on')
		else
			unknown('status', xbyte, 2)
	end
end;

procedure showbufs(a : word);

const
	bufsmax = 99;

var
	i : byte;
	xbool : boolean;
	xword1 : word;
	xword2 : word;
	xword3 : word;

begin
	i := 0;
	xword1 := memw[DOScseg : a];
	xword2 := memw[DOScseg : a + 2];
	xbool := false;
	repeat
		if i <= bufsmax then begin
			if xword1 < $FFFF then begin
				inc(i);
				xword3 := xword1;
				xword1 := memw[xword2 : xword3];
				xword2 := memw[xword2 : xword3 + 2]
			end else begin
				xbool := true;
				writeln(i)
			end
		end else begin
			xbool := true;
			dontknow
		end
	until xbool
end;
(*	BIX ms.dos/secrets #2  *)

procedure muxint(a : string; b : byte);

var
	xbyte : byte;

begin
	caption2(qindent + a);
	with regs do begin
		AX := b shl 8;
		intr($2F, regs);
		xbyte := AL;
		case xbyte of
			$00 : writeln('no; OK to install');
			$01 : writeln('no; not OK to install');
			$FF : writeln('yes')
			else
				unknown('status', xbyte, 2)
		end
	end
end;

function bin16(a : word) : string;

begin
	bin16 := bin8(hi(a)) + '_' + bin8(lo(a))
end;

procedure drvname(a : byte);

begin
	write(chr(ord('A') + a), ': ')
end;

procedure media(a : byte);

procedure diskette(a, b : byte);

begin
	writeln('diskette (', a, '-sided, ', b, ' sectors)')
end;

begin (* procedure media *)
	caption2(qindent + 'Media');
	case a of
		$FF : diskette(2, 8);
		$FE : diskette(1, 8);
		$FD : diskette(2, 9);
		$FC : diskette(1, 9);
		$F9 : diskette(2, 15);
		$F8 : writeln('fixed disk')
		else
			unknown('media', a, 2)
	end
end;

(*
**	end subprograms
*)
