(*
**	SYSID.PAS
**
**	Version 4.3
**
**	Usage:	[d:][path]SYSID
**
**	A system description for DOS-based PC/XT/AT- and PS/2-class machines.
**	SYSID generates 12 screens of information about the host system and runs
**	under DOS versions 3.0 and later.
**
**	My primary sources of ideas in SYSID were Ray Duncan's books Advanced
**	MS-DOS and IBM ROM BIOS.  The ideas of Terje Mathisen, Bob Smith, and
**	others appear in various places.
**
**	Some of the techniques SYSID uses are not documented or officially
**	supported by either IBM or Microsoft.  Where possible I have followed the
**	undocumented routine with a comment describing my source for the
**	technique.
**
**	SYSID was developed on an IBM PC with Turbo Pascal 5.0, Turbo Assembler
**	1.0, and DOS 3.30.  The source code has been split into three files:
**
**		SYSID.PAS	- this file
**		SYSID.INC	- an include file which contains all function and
**						procedure declarations
**		SYSID.ASM	- assembly language procedures
**
**	Known bugs:
**		1)	Page 1:  The CPU test for interrupts of multi-prefix string
**			instructions is reliable only on machines whose clock speeds are
**			less than about 15 MHz.  The 8086 and 8088 are the only CPU's that
**			don't handle such interrupts correctly, however, and they aren't
**			(to my knowledge) ever run at anything like 15 MHz.  (This 15 MHz
**			limit assumes that the timer tick interrupt occurs at the standard
**			rate of 18.2 Hz.  SYSID could check that, too, I suppose.)
**		2)	Page 1:  SYSID mistakenly identifies an 80387 as an 80287, for the
**			simple reason that I don't know of a software test to
**			differentiate between an 80287 and an 80387.
**		3)	Page 4:  The description of foreground color will not mention the
**			blinking attribute, even if it was enabled before you invoked
**			SYSID.
**		4)	Page 8:  SYSID used to report incorrectly the statuses of some of
**			the executable files which use the "multiplex interrupt"
**			(INT 2FH).  I have commented these status checks out of the source
**			code, determined to do battle with them another day.  Can anyone
**			supply the correct INT 2FH functions for these files?  Or are some
**			of them red herrings that simply check INT 2FH to see if *other*
**			files have been loaded (e.g. APPEND/ASSIGN)?
**
**	Both the source and object code of SYSID are hereby released into the
**	public domain.  Neither version carries any warranty, expressed or
**	implied, of merchantability or fitness for a particular purpose.
**
**	Comments, suggestions, and questions may be addressed to:
**		BIXMail: sjgrant
**		CompuServe: 71101,706
**
**	Steve Grant
**	Long Beach, CA
**	December 6, 1988
*)

(*$A-*)
(*$B-*)
(*$D-*)
(*$F-*)
(*$I-*)
(*$L+*)
(*$M 16384, 0, 655360*)
(*$N-*)
(*$R-*)
(*$S-*)
(*$V-*)

program SYSID;

uses
	crt,
	dos,
	graph;

const
	BIOScseg = $C000;
	BIOSdseg = $0040;
	BIOSext = $AA55;
	EMMint = $67;
	filesmax = 256;
	mouseint = $33;
	nuldev = 'NUL     ';
	opiret = $CF;
	PCROMseg = $F000;
	pgmax = 12;
	pchar : set of char = [' '..'~'];
	qEMMdrvr = 'EMMXXXX0';
	qhexpref = '$';
	qindent = '  ';
	qspace2 = '  ';
	qspace3 = '   ';
	qspace4 = '    ';
	qspace7 = '       ';
	qversion = 'Version 4.3';
	secsiz1 = 511;
	tickrate = 1193180;

var
	attrsave : byte;
	bootrec : array[0..secsiz1] of byte;
	bootstat : word;
	ccode : word;
	country : array[0..33] of byte;
	currdrv : byte;
	devofs : word;
	devseg : word;
	dirsep : set of char;
	DOScseg : word;
	DOScofs : word;
	DOSmem : longint;
	EMMarray : array[$000..$3FF] of word;
	equip : word;
	f : array[1..filesmax] of file;
	graphdriver : integer;
	graphmode : integer;
	header : array[0..17] of byte;
	i : word;
	intvec : array[$00..$FF] of pointer;
	j : word;
	lqindent : byte;
	osmajor : byte;
	osminor : byte;
	pg : 1..pgmax;
	regs : registers;
	search : searchrec;
	switchar : char;
	tlength : byte;
	topline : byte;
	twidth : byte;
	VGAbuf : array[$00..$10] of byte;
	vidpg : byte;
	x1 : byte;
	x2 : byte;
	xbool1 : boolean;
	xbool2 : boolean;
	xbool3 : boolean;
	xbyte1 : byte;
	xbyte2 : byte;
	xchar1 : char;
	xchar2 : char;
	xint1 : integer;
	xint2 : integer;
	xlongint : longint;
	xstring : string;
	xword1 : word;
	xword2 : word;
	xword3 : word;
	xword4 : word;
	xword5 : word;

(*$I SYSID.INC*)

begin
	xword1 := dosversion;
	osmajor := lo(xword1);
	osminor := hi(xword1);
	if osmajor >= 3 then begin
		attrsave := textattr;
		with regs do begin
			AH := $0F;
			intr($10, regs);
			twidth := AH;
			vidpg := BH
		end;
		detectgraph(graphdriver, graphmode);
		if (graphdriver = EGA) or (graphdriver = MCGA)
			or (graphdriver = VGA) then
			with regs do begin
				AX := $1130;
				BH := $00;
				intr($10, regs);
				tlength := DL + 1
			end
		else
			tlength := 25;
		with regs do begin
			intr($11, regs);
			equip := AX;
			intr($12, regs);
			DOSmem := longint(AX) shl 10;
			AH := $19;
			MSDOS(regs);
			currdrv := AL
		end;
		bootstat := diskread(currdrv, 0, 1, bootrec);
		with regs do begin
			AH := $34;
			MSDOS(regs);
			DOScseg := ES;
			DOScofs := BX
		end;
(*	BIX ms.dos/secrets #2  *)
		for i := $00 to $FF do
			getintvec(i, intvec[i]);
		intvec[$00] := saveint00;
		intvec[$02] := saveint02;
		intvec[$1B] := saveint1B;
		intvec[$23] := saveint23;
		intvec[$24] := saveint24;
		intvec[$34] := saveint34;
		intvec[$35] := saveint35;
		intvec[$36] := saveint36;
		intvec[$37] := saveint37;
		intvec[$38] := saveint38;
		intvec[$39] := saveint39;
		intvec[$3A] := saveint3A;
		intvec[$3B] := saveint3B;
		intvec[$3C] := saveint3C;
		intvec[$3D] := saveint3D;
		intvec[$3E] := saveint3E;
		intvec[$3F] := saveint3F;
		intvec[$75] := saveint75;
		with regs do begin
			AX := $3700;
			MSDOS(regs);
			switchar := chr(DL)
		end;
(*	BIX ms.dos/secrets #1130  *)
		dirsep := ['\'];
		if switchar <> '/' then
			dirsep := dirsep + ['/'];
		with regs do begin
			AH := $52;
			MSDOS(regs);
			devseg := ES;
			devofs := BX
		end;
		textbackground(black);
		window(1, 1, twidth, tlength);
		clrscr;
		textcolor(green);
		write('SYSID');
		textcolor(lightgray);
		write(' - System description for IBM PC''s and compatibles');
		rjustify(qversion);
		writeln;
		border;
		gotoxy(1, tlength - 1);
		border;
		writeln;
		write('Page ');
		x1 := wherex;
		write(pgmax, ' of ', pgmax);
		textcolor(green);
		rjustify('PgDn PgUp Home End Esc');
		x2 := wherex;
		pg := 1;
		lqindent := length(qindent);
		xbool1 := false;
		repeat
			gotoxy(x1, tlength);
			textcolor(lightgray);
			write(pg : 2);
			window(1, 3, twidth, tlength - 2);
			clrscr;
			case pg of
				1 : begin
					caption2('Machine type');
					with regs do begin
						AH := $C0;
						intr($15, regs);
						if nocarry then begin
							xword1 := memw[ES : BX + 2];
							if (xword1 = $00FC) or (xword1 = $01FC) then
								writeln('PC-AT 3x9')
							else if (xword1 = $00FB) or (xword1 = $01FB) then
								writeln('PC-XT/2')
							else if xword1 = $02FC then
								writeln('PC-XT/286')
							else if xword1 = $00F9 then
								writeln('PC-Convertible')
							else if xword1 = $00FA then
								writeln('PS/2 Model 30')
							else if xword1 = $04FC then
								writeln('PS/2 Model 50')
							else if xword1 = $05FC then
								writeln('PS/2 Model 60')
							else if (xword1 = $00F8) or (xword1 = $01F8) then
								writeln('PS/2 Model 80')
							else if xword1 = $06FC then
								writeln('7552 Gearbox')
							else
								unknown('machine - model/type word', xword1
									, 4);
							caption2(qindent + 'BIOS revision level');
							writeln(mem[ES : BX + 4]);
							caption2(qindent + 'Hardware configuration');
							writeln(bin8(mem[ES : BX + 5]))
						end else begin
							xbyte1 := mem[$FFFF : $000E];
							case xbyte1 of
								$FF : writeln('PC');
								$FE : writeln('PC-XT');
								$FD : writeln('PCjr');
								$FC : writeln('PC-AT')
								else
									unknown('machine - model byte', xbyte1, 2)
							end
						end
					end;
(*	Byte 12:12 p. 174  *)
					caption2('BIOS source');
					if BIOSscan(PCROMseg, $E000, $FFFF, xword1) then
						showBIOS(PCROMseg, xword1)
					else
						dontknow;
					caption2('BIOS date');
					i := $0005;
					xbool2 := false;
					xchar1 := chr(mem[$FFFF : i]);
					while (i < $0010) and (xchar1 in pchar) do begin
						xbool2 := true;
						write(xchar1);
						inc(i);
						xchar1 := chr(mem[$FFFF : i])
					end;
					if xbool2 then
						writeln
					else
						dontknow;
					caption2('BIOS extensions');
					xbool2 := false;
					for i := 0 to 7 do begin
						xword1 := BIOScseg + i shl 9;
						if (memw[xword1 : 0] = BIOSext) then begin
							if not xbool2 then begin
								writeln;
								caption1(qindent + 'Segment' + qspace3
									+ 'Copyright notice');
								writeln;
								xbool2 := true
							end;
							write(qindent, hex(xword1, 4), '   ', qspace3);
							if BIOSscan(xword1, 0, $1FFF, xword2) then
								showBIOS(xword1, xword2)
							else
								dontknow
						end
					end;
					if not xbool2 then
						writeln('(none)');
					caption2('CPU');
					xword1 := CPUID;
					xbyte1 := hi(xword1);
					case xbyte1 of
						$00 : writeln('8088');
						$01 : writeln('8086');
						$02 : writeln('V20');
						$03 : writeln('V30');
						$04 : writeln('80188');
						$05 : writeln('80186');
						$06 : writeln('80286');
						$07 : writeln('80386')
						else
							unknown('CPU type', xbyte1, 2)
					end;
					caption2(qindent + 'Interrupts acknowledged immediately'
						+ ' after segment register change');
					xbyte1 := lo(xword1);
					yesorno(xbyte1 = $01);
					writeln;
					caption2('Coprocessor');
					if equip and $0002 = $0002 then begin
						xlongint := NDPID;
						xbyte1 := xlongint and $00FF0000 shr 16;
						xword1 := xlongint and $0000FFFF;
						case xbyte1 of
							$00 : writeln('enabled but not present');
							$01 : showNDP('8087', xword1);
							$02 : showNDP('80287', xword1)
							else
								dontknow
						end
					end else
						writeln('not enabled')
				end;
				2 : begin
					caption2('Total conventional memory (bytes)');
					writeln(DOSmem : 6);
					caption2('Free conventional memory (bytes) ');
					writeln(DOSmem - longint(prefixseg) shl 4 : 6);
					caption2('Extended memory (bytes)        ');
					with regs do begin
						AH := $88;
						intr($15, regs);
						if nocarry then
							writeln(longint(AX) shl 10 : 8)
						else
							writeln('N/A' : 8)
					end;
					caption2('Expanded memory');
					if intinit(EMMint) then begin
						writeln;
						caption2(qindent + 'Interrupt vector');
						xlongint := longint(intvec[EMMint]);
						xword1 := xlongint shr 16;
						xword2 := xlongint and $0000FFFF;
						segofs(xword1, xword2);
						writeln;
						caption2(qindent + 'Driver');
						xstring := '';
						for i := $000A to $0011 do
							xstring := xstring
								+ showchar(chr(mem[xword1 : i]));
						write(xstring);
						if xstring = qEMMdrvr then begin
							writeln;
							caption2(qindent + 'Manager status');
							with regs do begin
								AH := $40;
								intr(EMMint, regs);
								if AH = $00 then
									writeln('OK')
								else
									EMMerr(AH);
								caption2(qindent + 'Page frame segment');
								AH := $41;
								intr(EMMint, regs);
								if AH = $00 then
									writeln(hex(BX, 4))
								else
									EMMerr(AH);
								caption2(qindent
									+ 'Total EMS memory (16K pages)');
								AH := $42;
								intr(EMMint, regs);
								if AH = $00 then
									writeln(DX : 3)
								else
									EMMerr(AH);
								caption2(qindent
									+ 'Free EMS memory (16K pages) ');
								if AH = $00 then
									writeln(BX : 3)
								else
									EMMerr(AH);
								caption2(qindent + 'EMM version');
								AH := $46;
								intr(EMMint, regs);
								if AH = $00 then
									writeln(AL shr 4, '.', AL and $0F)
								else
									EMMerr(AH);
								caption1(qindent + 'Handle' + qspace3
									+ '16K pages');
								writeln;
								AH := $4D;
								ES := seg(EMMarray);
								DI := ofs(EMMarray);
								intr(EMMint, regs);
								if AH = $00 then
									if BX > $0000 then begin
										topline:= 15;
										window(1 + lqindent, topline, twidth
											, tlength - 2);
										for i := 1 to BX do begin
											pause;
											writeln(hex(EMMarray[2 * i - 2]
												, 4), '  ', qspace3
												, EMMarray[2 * i - 1] : 3)
										end
									end else
										writeln(qindent
											, '(no active handles)')
								else
									EMMerr(AH)
							end
						end else
							writeln(' (unknown driver)')
					end else
						writeln('(none)')
				end;
				3 : begin
					caption1('MCB ' + qspace3 + 'PSP ' + qspace3 + 'Parent'
						+ qspace3 + '  Size' + qspace3 + 'Env' + qspace3
						+ 'Owner       ' + qspace3 + 'Interrupts');
					writeln;
					topline := 4;
					window(1, topline, twidth, tlength - 2);
					xword1 := memw[devseg : devofs - $02];
					xbool2 := false;
					repeat
						xbyte1 := mem[xword1 : $0000];
						xword2 := memw[xword1 : $0001];
						xword3 := memw[xword2 : $0016];
						pause;
						case xbyte1 of
							$4D : begin
								xword4 := memw[xword1 : $0003];
								showMCB(xword1, xword2, xword3, xword4);
								inc(xword1, 1 + xword4)
							end;
							$5A : begin
								xword4 := DOSmem shr 4 - xword1 - 1;
								showMCB(xword1, xword2, xword3, xword4);
								xbool2 := true
							end else begin
								unknown('MCB status', xbyte1, 2);
								xbool2 := true
							end
						end
					until xbool2
				end;
(*	PC Magazine 6:14 p.425  *)
				4 : begin
					with regs do begin
						AX := $1A00;
						intr($10, regs);
						if AL = $1A then begin
							showdisp('Active video subsystem  ', BL);
							showdisp('Inactive video subsystem', BH)
						end
					end;
					caption2('Initial video mode');
					case equip and $30 of
						$00 : writeln('No display');
						$10 : writeln('40 x 25 color');
						$20 : writeln('80 x 25 color');
						$30 : writeln('80 x 25 monochrome')
					end;
					caption2('Current video mode');
					xbyte1 := lo(lastmode);
					write(xbyte1, ' ');
					case xbyte1 of
						$00 : writeln('(40 x 25 b/w text)');
						$01 : writeln('(40 x 25 color text)');
						$02 : writeln('(80 x 25 b/w text)');
						$03 : writeln('(80 x 25 color text)');
						$04 : writeln('(320 x 200 4 colors)');
						$05 : writeln('(320 x 200 4 colors, no color burst)');
						$06 : writeln('(640 x 200 2 colors)');
						$07 : writeln('(MDA text)');
						$08 : writeln('(160 x 200 16 colors)');
						$09 : writeln('(320 x 200 16 colors)');
						$0A : writeln('(640 x 200 4 colors)');
						$0D : writeln('(320 x 200 16 colors)');
						$0E : writeln('(640 x 200 16 colors)');
						$0F : writeln('(640 x 350 monochrome)');
						$10 : writeln('(640 x 350 16 colors)');
						$11 : writeln('(640 x 480 2 colors)');
						$12 : writeln('(640 x 480 16 colors)');
						$13 : writeln('(640 x 480 256 colors)')
						else
							unknown('video mode', xbyte1, 2)
					end;
(*	Byte 12:12 p. 176D  *)
					caption2('Current display page');
					writeln(vidpg);
					caption2('Valid graphics modes');
					getmoderange(graphdriver, xint1, xint2);
					writeln(xint1, '-', xint2);
					caption2('Video buffer (offset)');
					writeln(hex(memw[BIOSdseg : $004E], 4));
(*	PC Magazine 6:8 p.290  *)
					caption2('Video buffer size (bytes)');
					writeln(memw[BIOSdseg : $004C]);
(*	PC Magazine 6:8 p.290  *)
					caption2('Active display port');
					xword1 := memw[BIOSdseg : $0063];
					write(qhexpref, hex(xword1, 3), ' ');
					if xword1 = $3B4 then
						writeln('(monochrome)')
					else if xword1 = $3D4 then
						writeln('(color)')
					else
						dontknow;
(*	PC Magazine 6:8 p.290  *)
					caption2('CRT mode register');
					writeln(qhexpref, hex(mem[BIOSdseg : $0065], 2));
(*	PC Magazine 6:8 p.290  *)
					caption2('Current palette');
					writeln(qhexpref, hex(mem[BIOSdseg : $0066], 2), qindent);
(*	PC Magazine 6:8 p.290  *)
					caption2('Colors');
					caption1('');
					for i := black to white do begin
						textcolor(i);
						write('')
					end;
					caption1('');
					writeln;
					caption2('Current colors');
					if (attrsave and $80) = $80 then
						write('blinking ');
					showcolor(attrsave and $0F);
					write(' on ');
					showcolor(attrsave and $70 shr 4);
					writeln;
					caption2('Text rows');
					writeln(tlength);
					caption2('Text columns');
					writeln(twidth);
					if graphdriver in [EGA, MCGA, VGA] then begin
						caption2('Scan lines/character');
						with regs do begin
							AX := $1130;
							BH := $00;
							intr($10, regs);
							writeln(CX)
						end
					end;
					caption2('Cursor scan lines');
					with regs do begin
						AH := $03;
						BH := vidpg;
						intr($10, regs);
						writeln(CH, '-', CL)
					end
				end;
				5 : begin
					caption2('Display adapter');
					case graphdriver of
						CGA : begin
							writeln('CGA');
							captfont
						end;
						MCGA : begin
							writeln('MCGA');
							captfont;
							showfont($01);
							showfont($03);
							showfont($04);
							showfont($06)
						end;
						EGA..EGAmono : begin
							writeln('EGA');
							captfont;
							showfont($01);
							showfont($02);
							showfont($03);
							showfont($04);
							showfont($05);
							int101210;
							xbyte1 := mem[BIOSdseg : $0087];
							caption1('Screen buffer ');
							case xbyte1 and $80 of
								$00 : write('cleared');
								$80 : write('preserved')
							end;
							caption1(' during last mode change');
							writeln;
							caption2('EGA active');
							yesorno(xbyte1 and $08 = $00);
							writeln;
							caption2('Wait for display enable');
							yesorno(xbyte1 and $04 = $04);
							writeln;
							caption2('CGA cursor emulation');
							yesorno(xbyte1 and $01 = $00);
							writeln;
(*	PC Magazine 6:12 p.326  *)
							caption2('Save area                    ');
							xword1 := memw[BIOSdseg : $00AA];
							xword2 := memw[BIOSdseg : $00A8];
							segofs(xword1, xword2);
							writeln;
(*	PC Tech Journal 3:4 p.65  *)
							caption2('Video parameter table        ');
							segofs(memw[xword1 : xword2 +  2]
								, memw[xword1 : xword2]);
							writeln;
							caption2('Dynamic save area            ');
							xword3 := memw[xword1 : xword2 +  6];
							xword4 := memw[xword1 : xword2 +  4];
							if (xword3 > $0000) or (xword4 > $0000) then begin
								segofs(xword3, xword4);
								writeln
							end else
								writeln('(none)');
							caption2('Auxiliary character generator');
							xword3 := memw[xword1 : xword2 + 10];
							xword4 := memw[xword1 : xword2 +  8];
							if (xword3 > $0000) or (xword4 > $0000) then begin
								segofs(xword3, xword4);
								writeln
							end else
								writeln('(none)');
							caption2('Graphics mode auxiliary table');
							xword3 := memw[xword1 : xword2 + 14];
							xword4 := memw[xword1 : xword2 + 12];
							if (xword3 > $0000) or (xword4 > $0000) then
								segofs(xword3, xword4)
							else
								write('(none)')
(*	PC Tech Journal 3:4 p.67  *)
						end;
						hercmono : begin
							writeln('Hercules or MDA');
							captfont
						end;
						ATT400 : begin
							writeln('AT&T 400');
							captfont
						end;
						VGA : begin
							writeln('VGA');
							captfont;
							showfont($01);
							showfont($02);
							showfont($03);
							showfont($04);
							showfont($05);
							showfont($06);
							showfont($07);
							int101210;
							with regs do begin
								AX := $1009;
								ES := seg(VGAbuf);
								DX := ofs(VGAbuf);
								intr($10, regs)
							end;
							caption2('Palette registers');
							for i := $00 to $0F do
								write(hex(VGAbuf[i], 2), ' ');
							writeln;
							caption2('Border color');
							writeln(hex(VGAbuf[$10], 2));
							caption2('Color page');
							with regs do begin
								AX := $101A;
								intr($10, regs);
								writeln(qhexpref, hex(BH, 2));
								caption2('Paging mode');
								case BL of
									$00 : writeln('4 pages of 64 registers');
									$01 : writeln('16 pages of 16 registers')
									else
										dontknow
								end
							end
						end;
						PC3270 : begin
							writeln('3270 PC');
							captfont
						end else
							unknown('adapter', graphdriver, 4)
					end
				end;
				6 : begin
					caption2('Keyboard');
					writeln;
					caption2(qindent + 'BIOS support for enhanced keyboard');
					with regs do begin
						AX := $1200 + xbyte1 xor $FF;
						intr($16, regs);
						if AL = xbyte1 then begin
							writeln('yes');
							caption2(qindent + 'Enhanced keyboard present');
							yesorno(mem[BIOSdseg : $0096] and $10 = $10);
							writeln
						end else
							writeln('no');
(*	PC Magazine 6:15 p.378  *)
						caption2(qindent + 'Insert');
						AH := $02;
						intr($16, regs);
						offoron(AL and $80 = $80);
						caption2(qspace7 + 'Caps Lock');
						offoron(AL and $40 = $40);
						caption2(qspace7 + 'Num Lock');
						offoron(AL and $20 = $20);
						caption2(qspace7 + 'Scroll Lock');
						offoron(AL and $10 = $10);
						writeln
					end;
					caption2(qindent + 'Buffer');
					xword1 := memw[BIOSdseg : $0080];
					segofs(BIOSdseg, xword1);
					xword2 := memw[BIOSdseg : $0082];
					writeln('-', hex(xword2, 4));
(*	PC Magazine 6:8 p.290  *)
					caption2(qindent + 'Buffer size (keystrokes)');
					writeln((xword2 - xword1) shr 1 - 1);
					caption2('Printers');
					xbyte1 := equip and $C000 shr 14;
					writeln(xbyte1);
					if xbyte1 > 0 then begin
						caption1(qindent + 'Device' + qspace2 + 'Addr'
							+ qspace2 + 'Timeout' + qspace2 + 'Busy'
							+ qspace2 + 'Ack' + qspace2 + 'Paper out'
							+ qspace2 + 'Selected' + qspace2 + 'I/O error'
							+ qspace2 + 'Timed out');
						writeln;
						for i := 1 to xbyte1 do begin
							write(qindent, 'LPT', i, '  ', qspace2, qhexpref
								, hex(memw[BIOSdseg : 2 * i + 6], 3), qspace2
								, mem[BIOSdseg : $0077 + i] : 3, '    '
								, qspace2);
							with regs do begin
								AH := $02;
								DX := i - 1;
								intr($17, regs);
								yesorno(AH and $80 = $00);
								write(' ', qspace2);
								yesorno(AH and $40 = $40);
								write(qspace2);
								yesorno(AH and $20 = $20);
								write('      ', qspace2);
								yesorno(AH and $10 = $10);
								write('     ', qspace2);
								yesorno(AH and $08 = $08);
								write('      ', qspace2);
								yesorno(AH and $01 = $01);
								writeln
							end
						end
					end;
					caption2('Internal modem/serial printer');
					yesorno(equip and $2000 = $2000);
					writeln;
					caption2('Game port');
					yesorno(equip and $1000 = $1000);
					writeln;
					caption2('Serial ports');
					xbyte1 := equip and $0E00 shr 9;
					writeln(xbyte1);
					if xbyte1 > 0 then begin
						if xbyte1 > 4 then
							xbyte1 := 4;
						caption1(qindent + 'Device' + qspace3 + 'Addr'
							+ qspace3 + 'Timeout' + qspace3 + 'RLSD'
							+ qspace3 + 'RI ' + qspace3 + 'DSR' + qspace3
							+ 'CTS' + qspace3 + 'dRLSD' + qspace3 + '-dRI'
							+ qspace3 + 'dDSR' + qspace3 + 'dCTS');
						writeln;
						for i := 1 to xbyte1 do begin
							write(qindent, 'COM', i, '  ', qspace3, qhexpref
								, hex(memw[BIOSdseg : 2 * i - 2], 3), qspace3
								, mem[BIOSdseg : $007B + i] : 3, '    '
								, qspace3);
							with regs do begin
								AH := $03;
								DX := i - 1;
								intr($14, regs);
								yesorno(AL and $80 = $80);
								write(' ', qspace3);
								yesorno(AL and $40 = $40);
								write(qspace3);
								yesorno(AL and $20 = $20);
								write(qspace3);
								yesorno(AL and $10 = $10);
								write(qspace3);
								yesorno(AL and $08 = $08);
								write('  ', qspace3);
								yesorno(AL and $04 = $04);
								write(' ', qspace3);
								yesorno(AL and $02 = $02);
								write(' ', qspace3);
								yesorno(AL and $01 = $01);
								writeln
							end
						end
					end;
					caption2('Mouse');
					if byte(intvec[mouseint]^) <> opiret then begin
						writeln('yes');
						caption2(qindent + 'Status');
						with regs do begin
							AX := 0;
							intr(mouseint, regs);
							if AX = $FFFF then begin
								writeln('present');
								caption2(qindent + 'Buttons');
								writeln(BX)
							end else if AX = $0000 then
								writeln('not present')
							else
								unknown('status', AX, 4)
						end
					end else
						writeln('no')
(*	PC Magazine 6:13 p.420  *)
				end;
				7 : begin
					window(1, 3, twidth shr 1, tlength - 2);
					with regs do begin
						AX := $3800;
						DS := seg(country);
						DX := ofs(country);
						MSDOS(regs);
						ccode := BX
					end;
					caption2('DOS version');
					showvers;
					caption2('System date');
					getdate(xword1, xword2, xword3, xword4);
					if xword4 = 0 then
						write('Sunday')
					else if xword4 = 1 then
						write('Monday')
					else if xword4 = 2 then
						write('Tuesday')
					else if xword4 = 3 then
						write('Wednesday')
					else if xword4 = 4 then
						write('Thursday')
					else if xword4 = 5 then
						write('Friday')
					else if xword4 = 6 then
						write('Saturday')
					else
						write('(', hex(xword4, 4), ')');
					write(', ');
					xword5 := cbw(country[1], country[0]);
					xchar1 := chr(country[11]);
					if xword5 = $0000 then
						writeln(xword2, xchar1, xword3, xchar1, xword1)
					else if xword5 = $0001 then
						writeln(xword3, xchar1, xword2, xchar1, xword1)
					else if xword5 = $0002 then
						writeln(xword1, xchar1, xword2, xchar1, xword3)
					else
						writeln(xword2, xchar1, xword3, xchar1, xword1);
					caption2('System time');
					gettime(xword1, xword2, xword3, xword4);
					zeropad(xword1);
					write(chr(country[13]));
					zeropad(xword2);
					write(chr(country[13]));
					zeropad(xword3);
					write(chr(country[9]));
					zeropad(xword4);
					writeln;
					caption2('Command load paragraph');
					writeln(hex(prefixseg, 4));
					caption2('Ctrl-C check');
					getcbreak(xbool2);
					offoron(xbool2);
					writeln;
					caption2('Disk verify');
					getverify(xbool2);
					offoron(xbool2);
					writeln;
					caption2('Switch prefix character');
					writeln(switchar);
(*	BIX ms.dos/secrets #1130  *)
					caption2('\DEV\ prefix for devices');
					with regs do begin
						AX := $3702;
						MSDOS(regs);
						if DL = $00 then
							writeln('required')
						else
							writeln('optional')
					end;
(*	BIX ms.dos/secrets #1130  *)
					caption2('Reset boot');
					xword1 := memw[BIOSdseg : $72];
					if xword1 = $0000 then
						writeln('cold')
					else if xword1 = $1234 then
						writeln('bypass memory test')
					else if xword1 = $4321 then
						writeln('preserve memory')
					else if xword1 = $5678 then
						writeln('system suspended')
					else if xword1 = $9ABC then
						writeln('manufacturing test mode')
					else if xword1 = $ABCD then
						writeln('system POST loop mode')
					else
						unknown('flag', xword1, 4);
(*	Byte 12:12 p.178  *)
					caption2('DOS busy flag');
					segofs(DOScseg, DOScofs);
					writeln;
					caption2('Printer echo');
					case osmajor of
						3 : case osminor div 10 of
							0 : dontknow;
							1..3 : showecho($02AC)
							else
								dontknow
						end else
							dontknow
					end;
(*	BIX ms.dos/secrets #501  *)
					caption2('PrtSc status');
					xbyte1 := mem[BIOSdseg : $0100];
					case xbyte1 of
						$00 : writeln('ready');
						$01 : writeln('busy');
						$FF : writeln('error on last PrtSc')
						else
							unknown('status', xbyte1, 2)
					end;
(*	PC Magazine 6:20 p.412  *)
					caption2('Memory allocation');
					with regs do begin
						AX := $5800;
						MSDOS(regs);
						if AX = $0000 then
							writeln('first fit')
						else if AX = $0001 then
							writeln('best fit')
						else if AX = $0002 then
							writeln('last fit')
						else
							unknown('strategy', AX, 4)
					end;
					caption2('DOS buffers');
					case osmajor of
						3 : case osminor div 10 of
							0 : showbufs($013F);
							1..3 : showbufs($0038)
							else
								dontknow
						end
						else
							dontknow
					end;
					caption2('File handle table ');
					xword1 := memw[prefixseg : $0036];
					xword2 := memw[prefixseg : $0034];
					segofs(xword1, xword2);
					writeln;
					caption2('File handle table length');
					writeln(mem[prefixseg : $0032] : 3);
					caption2('File handles used       ');
					i := 0;
					while mem[xword1 : xword2] < $FF do begin
						inc(i);
						inc(xword2)
					end;
					writeln(i : 3);
					caption1('File handles free');
					findfirst('*.*', archive, search);
					if doserror = 0 then begin
						i := 0;
						xbool2 := false;
						repeat
							if i < filesmax then begin
								assign(f[i + 1], search.name);
								reset(f[i + 1]);
								if ioresult = 0 then
									inc(i)
								else begin
									xbool2 := true;
									caption2('       ');
									writeln(i : 3)
								end
							end else begin
								xbool2 := true;
								caption2('');
								dontknow
							end
						until xbool2;
						for j := 1 to i do
							close(f[j])
					end else begin
						caption2('');
						dontknow
					end;
					caption2('Global code page');
					with regs do begin
						AX := $6601;
						MSDOS(regs);
						if AL = $01 then begin
							writeln;
							caption2(qindent + 'Active ');
							writeln(BX : 5);
							caption2(qindent + 'Default');
							write(DX : 5)
						end else
							writeln('N/A')
					end;
(*	PC Magazine 6:13 p. 181  *)
					window(1 + twidth shr 1, 3, twidth, tlength - 2);
					caption2('Country code');
					writeln(ccode);
					caption2('Thousands separator character');
					writeln(chr(country[7]));
					caption2('Decimal separator character');
					writeln(chr(country[9]));
					caption2('Data-list separator character');
					writeln(chr(country[22]));
					caption2('Date format');
					xword1 := cbw(country[1], country[0]);
					xchar1 := chr(country[11]);
					if xword1 = $0000 then
						writeln('USA (mm', xchar1, 'dd', xchar1, 'yy)')
					else if xword1 = $0001 then
						writeln('Europe (dd', xchar1, 'mm', xchar1, 'yy)')
					else if xword1 = $0002 then
						writeln('Japan (yy', xchar1, 'mm', xchar1, 'dd)')
					else
						unknown('format', xword1, 4);
					caption2(qindent + 'Separator character');
					writeln(xchar1);
					caption2('Time format');
					if (country[17] and $01) = $00 then
						write('12')
					else
						write('24');
					writeln('-hour');
					caption2(qindent + 'Separator character');
					writeln(chr(country[13]));
					caption2('Currency format');
					writeln;
					caption2(qindent + 'Currency symbol string');
					i := 2;
					xchar1 := chr(country[i]);
					while xchar1 > #0 do begin
						write(xchar1);
						inc(i);
						xchar1 := chr(country[i])
					end;
					writeln;
					caption1(qindent + 'Currency symbol ');
					if (country[15] and $01) = $00 then
						write('precedes')
					else
						write('follows');
					caption1(' value');
					writeln;
					caption2(qindent + 'Spaces between symbol and value');
					writeln(country[15] shr 1 and 1);
					caption2(qindent + 'Digits after decimal');
					writeln(country[16]);
					caption2('Case map call address');
					segofs(cbw(country[21], country[20]), cbw(country[19]
						, country[18]));
					writeln
				end;
				8 : begin
					caption2('Multiplex interrupt ($2F)');
					writeln;
					muxint('PRINT          ', $01);
(*	Byte 12:12 p. 176C  *)
					muxint('ASSIGN         ', $06);
(*
**	Byte 12:12 p. 176C, Duncan, and many others, all of whom mistakenly give
**	AH = $02
*)
(*
					muxint('DRIVER.SYS     ', $08);
*)
					muxint('SHARE          ', $10);
(*	Byte 12:12 p. 176C  *)
(*
					muxint('FASTOPEN       ', $12);
*)
					muxint('NLSFUNC        ', $14);
					muxint('GRAFTABL       ', $B0);
(*
					muxint('DISPLAY.SYS    ', $B0);
*)
					muxint('APPEND         ', $B7);
(*	Byte 12:12 p. 176C  *)
(*
					muxint('KEYB           ', $B8);
*)
					muxint('NETBIOS append ', $87);
					muxint('NETBIOS network', $88);
(*	Byte 12:12 p. 180.  PC Tech Journal 3:11 p.104 gives AH = $BB  *)
					caption2('Environment');
					writeln;
					topline := 13;
					window(1 + lqindent, topline, twidth, tlength - 2);
					for i := 1 to envcount do begin
						pause;
						writeln(envstr(i))
					end
				end;
				9 : begin
					caption1('Device  ' + qspace4 + 'Units' + qspace4
						+ 'Header   ' + qspace4 + 'Attributes         '
						+ qspace4 + 'Strategy ' + qspace4 + 'Interrupt');
					writeln;
					if scan(nuldev, devseg, devofs, devofs + $100, xword1)
						then begin
						xword2 := devseg;
						xword3 := xword1 - 10;
						topline := 4;
						window(1, topline, twidth, tlength - 2);
						while xword3 < $FFFF do begin
							pause;
							for i := 0 to 17 do
								header[i] := mem[xword2 : xword3 + i];
							if header[5] and $80 = 0 then
								write('        ', qspace4, header[10] : 5)
							else begin
								for i := 10 to 17 do
									write(showchar(chr(header[i])));
								write(qspace4, '     ')
							end;
							write(qspace4);
							segofs(xword2, xword3);
							write(qspace4, bin16(cbw(header[5], header[4]))
								, qspace4);
							segofs(xword2, cbw(header[7], header[6]));
							write(qspace4);
							segofs(xword2, cbw(header[9], header[8]));
							writeln;
							xword2 := cbw(header[3], header[2]);
							xword3 := cbw(header[1], header[0])
						end
					end else
						writeln('(can''t find NUL device header)')
(*	BIX ms.dos/secrets #1032  *)
				end;
				10 : begin
					caption2('Logical drives');
					with regs do begin
						for xchar1 := 'A' to 'Z' do begin
							AH := $0E;
							DL := ord(xchar1) - ord('A');
							MSDOS(regs);
							AH := $19;
							MSDOS(regs);
							if AL = DL then
								drvname(AL)
						end;
						writeln;
						AH := $0E;
						DL := currdrv;
						MSDOS(regs)
					end;
					caption2('Diskette drives');
					if equip and $0001 = $0001 then
						writeln(1 + equip and $00C0 shr 6)
					else
						writeln(0);
					xword1 := longint(intvec[$1E]) shr 16;
					xword2 := longint(intvec[$1E]) and $0000FFFF;
					caption2(qindent + 'Sectors/track');
					writeln(mem[xword1 : xword2 + 4]);
					caption2(qindent + 'Bytes/sector');
					writeln(mem[xword1 : xword2 + 3] shl 8);
					caption2(qindent + 'On time (ms)');
					writeln(125 * mem[xword1 : xword2 + 10]);
					caption2(qindent + 'Off time (s)');
					writeln(longint(mem[xword1 : xword2 + 2]) shl 16
						/ tickrate : 0 : 1);
					caption2(qindent + 'Settling time (ms)');
					writeln(mem[xword1 : xword2 + 9]);
					caption1(qindent + 'Single drive is now ');
					xbyte1 := mem[BIOSdseg : $0104];
					if xbyte1 <= ord('Z') - ord('A') then begin
						drvname(xbyte1);
						writeln
					end else if xbyte1 = $FF then
						writeln('N/A')
					else
						unknown('status', xbyte1, 2);
(*	Byte 12:12 p.178  *)
					caption2('Current drive and path');
					getdir(0, xstring);
					writeln(xstring);
					caption2(qindent + 'Volume label');
					findfirst('\*.*', volumeid, search);
					if doserror = 0 then
						writeln(search.name)
					else
						writeln('(none)');
					with regs do begin
						AH := $1B;
						MSDOS(regs);
						media(mem[DS : BX]);
						caption2(qindent + 'Clusters');
						writeln(DX);
						caption2(qindent + 'Sectors/cluster');
						writeln(AL);
						caption2(qindent + 'Bytes/sector');
						writeln(CX)
					end;
					caption2(qindent + 'Total space (bytes)');
					xlongint := disksize(0);
					if xlongint <> -1 then
						writeln(xlongint : 8)
					else
						writeln('(invalid drive)');
					caption2(qindent + 'Free space (bytes) ');
					xlongint := diskfree(0);
					if xlongint <> -1 then
						writeln(xlongint : 8)
					else
						writeln('(invalid drive)')
				end;
				11 : begin
					caption2('BIOS disk parameters');
					writeln;
					caption1(qindent + 'Unit #' + qspace3
						+ 'Type                 ' + qspace3 + 'Drives'
						+ qspace3 + 'Heads' + qspace3 + 'Cylinders' + qspace3
						+ 'Sectors/track');
					writeln;
					with regs do begin
						for i := $00 to $7F do begin
							AH := $08;
							DL := i;
							intr($13, regs);
							if nocarry and (BL <> $00) then begin
								write(qindent, i : 3, '   ', qspace3);
								case BL of
									$01 : write(' 360KB  40 track  5"');
									$02 : write(' 1.2MB  80 track  5"');
									$03 : write(' 720KB  80 track  3"');
									$04 : write('1.44MB  80 track  3"')
									else
										write('unknown diskette (', hex(BL, 2)
											, ')')
								end;
								writeln(qspace3, DL : 3, '   ', qspace3
									, DH + 1 : 3, '  ', qspace3
									, cbw((CL and $C0) shr 6, CH) + 1 : 4
									, '     ', qspace3, CL and $3F : 2)
							end
						end;
						for i := $80 to $FF do begin
							AH := $08;
							DL := i;
							intr($13, regs);
							if nocarry then begin
								write(qindent, i : 3, '   ', qspace3);
								write('fixed disk           ');
								writeln(qspace3, DL : 3, '   ', qspace3
									, DH + 1 : 3, '  ', qspace3
									, cbw((CL and $C0) shr 6, CH) + 1 : 4
									, '     ', qspace3, CL and $3F : 2)
							end
						end
					end
				end;
				12 : begin
					window(1, 3, twidth, tlength - 2);
					caption1('DOS disk parameter block for ');
					with regs do begin
						AH := $32;
						DX := $0000;
						MSDOS(regs);
						if AL = $00 then begin
							drvname(mem[DS : BX]);
							writeln;
							media(mem[DS : BX + $16]);
							caption2(qindent + 'Sectors/cluster');
							writeln(mem[DS : BX + $04] + 1);
							caption2(qindent + 'Bytes/sector');
							writeln(memw[DS : BX + $02]);
							caption2(qindent + 'Reserved sectors');
							writeln(memw[DS : BX + $06]);
							caption2(qindent + 'FAT''s');
							writeln(mem[DS : BX + $08]);
							caption2(qindent + 'Sectors/FAT');
							writeln(mem[DS : BX + $0F]);
							caption2(qindent + 'Root directory entries');
							writeln(memw[DS : BX + $09]);
							writeln;
							caption2(qindent + 'Device header');
							segofs(memw[DS : BX + $14], memw[DS : BX + $12]);
							writeln;
							caption2(qindent + 'Unit within driver');
							writeln(mem[DS : BX + $01] + 1);
							caption2(qindent + 'Clusters');
							writeln(memw[DS : BX + $0D] - 1);
							caption2(qindent + 'Cluster to sector shift');
							writeln(mem[DS : BX + $05]);
							caption2(qindent + 'Root directory sector');
							writeln(memw[DS : BX + $10]);
							caption2(qindent + 'First data sector');
							writeln(memw[DS : BX + $0B]);
							caption2(qindent + 'Next DPB');
							segofs(memw[DS : BX + $1A], memw[DS : BX + $18]);
							writeln
						end else
							writeln(qindent, '(DOS function 32H error'
								, hex(AL, 2), ')')
					end;
(*	BIX ms.dos/secrets #678  *)
					window(1 + twidth shr 1, 3, twidth, tlength - 2);
					caption1('Boot record of ');
					drvname(currdrv);
					writeln;
					if bootstat = $0000 then begin
						media(bootrec[$15]);
						caption2(qindent + 'Sectors/cluster');
						writeln(bootrec[$0D]);
						caption2(qindent + 'Bytes/sector');
						writeln(cbw(bootrec[$0C], bootrec[$0B]));
						caption2(qindent + 'Reserved sectors');
						writeln(cbw(bootrec[$0F], bootrec[$0E]));
						caption2(qindent + 'FAT''s');
						writeln(bootrec[$10]);
						caption2(qindent + 'Sectors/FAT');
						writeln(cbw(bootrec[$17], bootrec[$16]));
						caption2(qindent + 'Root directory entries');
						writeln(cbw(bootrec[$12], bootrec[$11]));
						writeln;
						caption2(qindent + 'Heads');
						writeln(cbw(bootrec[$1B], bootrec[$1A]));
						caption2(qindent + 'Total sectors');
						writeln(cbw(bootrec[$14], bootrec[$13]));
						caption2(qindent + 'Sectors/track');
						writeln(cbw(bootrec[$19], bootrec[$18]));
						caption2(qindent + 'Hidden sectors');
						writeln(cbw(bootrec[$1D], bootrec[$1C]));
						caption2(qindent + 'OEM name and version');
						for i := $03 to $0A do
							write(showchar(chr(bootrec[i])));
						writeln
					end else begin
						writeln(qindent, 'Couldn''t read boot record');
						write(qindent);
						xbyte1 := hi(bootstat);
						case xbyte1 of
							$80 : writeln('Attachment failed to respond');
							$40 : writeln('Seek operation failed');
							$20 : writeln('Controller failed');
							$10 : writeln('Data error (bad CRC)');
							$08 : writeln('DMA failure');
							$04 : writeln('Sector not found');
							$03 : writeln('Write-protect fault');
							$02 : writeln('Bad address mark');
							$01 : writeln('Bad command');
							$00 :
							else
								unknown('error', xbyte1, 2)
						end;
						write(qindent);
						xbyte1 := lo(bootstat);
						case xbyte1 of
							$00 : writeln('Write-protect error');
							$01 : writeln('Unknown unit');
							$02 : writeln('Drive not ready');
							$03 : writeln('Unknown command');
							$04 : writeln('Data error (bad CRC)');
							$05 : writeln('Bad request structure length');
							$06 : writeln('Seek error');
							$07 : writeln('Unknown media type');
							$08 : writeln('Sector not found');
							$09 : writeln('Printer out of paper');
							$0A : writeln('Write fault');
							$0B : writeln('Read fault');
							$0C : writeln('General failure')
							else
								unknown('error', xbyte1, 2)
						end
					end
				end
			end;
			window(1, 1, twidth, tlength);
			gotoxy(x2, tlength);
			xbool2 := false;
			repeat
				repeat
				until keypressed;
				xchar1 := readkey;
				if keypressed then
					xchar2 := readkey
				else
					xchar2 := #0;
				if (xchar1 = #27) and (xchar2 = #0) then begin
					xbool2 := true;
					xbool1 := true
				end else if (xchar1 = #0) and (xchar2 = #71) and (pg > 1) then
					begin
					xbool2 := true;
					pg := 1
				end else if (xchar1 = #0) and (xchar2 = #73) and (pg > 1) then
					begin
					xbool2 := true;
					dec(pg)
				end else if (xchar1 = #0) and (xchar2 = #79) and (pg < pgmax)
					then begin
					xbool2 := true;
					pg := pgmax
				end else if (xchar1 = #0) and (xchar2 = #81) and (pg < pgmax)
					then begin
					xbool2 := true;
					inc(pg)
				end else begin
					sound(220);
					delay(100);
					nosound
				end
			until xbool2
		until xbool1;
		textattr := attrsave;
		clrscr
	end else begin
		writeln;
		writeln('SYSID requires DOS version 3.0 or later');
		write('Your DOS version is ');
		showvers
	end
end.
